<?php

namespace Drupal\simple_survey\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\simple_survey\BigQueryConnect;
use Drupal\simple_survey\Service\ResponseExportService;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\simple_survey\Entity\SimpleSurveyInterface;

/**
 * Class SimpleSurveyController.
 *
 * @package Drupal\simple_survey\Controller
 */
class SimpleSurveyController extends ControllerBase {

  /**
   * The modules path.
   *
   * @var string
   */
  private $modulePath = '';

  /**
   * The modules machine name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Drupal\Core\Logger\LoggerChannelFactory definition.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected $loggerFactory;

  /**
   * Drupal date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * Response export service.
   *
   * @var \Drupal\simple_survey\BigQueryService
   */
  protected $bigQuery;

  /**
   * Response export service.
   *
   * @var \Drupal\simple_survey\ResponseExportService
   */
  protected $responseExport;

  /**
   * The Drupal Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Simple Survery Controller constructor.
   *
   * @todo Look into creating a service to handle DI. Doing a janky call in feedbackjs.module
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The Drupal messenger service.
   * @param Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The Drupal date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\simple_survey\BigQueryConnect $big_query
   *   The BigQuery Connection service.
   * @param \Drupal\simple_survey\ResponseExportService $response_export_service
   *   The response export service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The Drupal time service.
   */
  public function __construct(
    MessengerInterface $messenger,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $logger_factory,
    BigQueryConnect $big_query,
    ResponseExportService $response_export_service,
    TimeInterface $time) {
    $this->messenger = $messenger;
    $this->dateFormatter = $dateFormatter;
    $this->loggerFactory = $logger_factory->get($this->moduleName);
    $this->bigQuery = $big_query;
    $this->responseExport = $response_export_service;
    $this->time = $time;

    $this->modulePath = drupal_get_path('module', $this->moduleName);
  }

  /**
   * Dependency Injection.
   *
   * @inheritDoc
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('date.formatter'),
      $container->get('logger.factory'),
      $container->get('simple_survey.big_query'),
      $container->get('simple_survey.response_export'),
      $container->get('datetime.time')
    );
  }

  /**
   * Displays the response as CSV.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Returns a JSON Response.
   */
  public function responseOutput(SimpleSurveyInterface $simple_survey = NULL) {
    $fileTime = $this->dateFormatter->format($this->time->getRequestTime(), 'custom', 'YdmHis');

    $headerResponse = new Response();
    $headerResponse->headers->set('Content-Type', 'application/csv');
    $headerResponse->headers->set('Content-Disposition', 'attachment; filename="survey-export-' . $fileTime . '.csv"');
    $headerResponse->send();

    $this->responseExport->convertDataToCsv(FALSE);

    return $headerResponse;
  }

  /**
   * Submits all data to a bigQuery table via cron.
   */
  public function bigQueryCronUpdate() {
    if (!$this->bigQuery->checkConfigured()) {
      $this->loggerFactory->alert('BigQuery is not configured.');
    }
    else {
      $csv = $this->responseExport->convertDataToCsv();

      $updated = $this->bigQuery->updateTable('survey_tool', 'survey_response', $csv);

      // Check if the job has errors.
      if ($updated) {
        $this->loggerFactory->info('Completed Updating BigQuery Table');
      }
    }
  }

  /**
   * Manual submission to BigQuery.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Returns response object.
   */
  public function bigQueryManualUpdate() {
    $headerResponse = new Response();

    if (!$this->bigQuery->checkConfigured()) {
      $this->messenger->addMessage('BigQuery is not configured.');
      $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
      $headerResponse->setContent('BigQuery is not enabled');

      return $headerResponse;
    }

    $csv = $this->responseExport->convertDataToCsv();

    $updated = $this->bigQuery->updateTable('survey_tool', 'survey_response', $csv);

    // Check if the job has errors.
    if (!$updated) {
      $this->messenger->addMessage($this->t('Data could not be sent.'));

      $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
      $headerResponse->setContent('Invalid Request Received');

      return $headerResponse;
    }
    else {
      $this->messenger->addMessage($this->t('Data sent to BigQuery.'));
      $headerResponse->setStatusCode(Response::HTTP_OK);
      $headerResponse->setContent('Valid Request');

      return $headerResponse;
    }
  }

}
