<?php

namespace Drupal\simple_survey\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\simple_survey\Entity\SurveyResponse;
use Drupal\simple_survey\Entity\SurveyAnswer;
use Drupal\Core\Entity\EntityStorageException;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Cookie;
use Snipe\BanBuilder\CensorWords;
use Drupal\simple_survey\BigQueryConnect;
use Drupal\simple_survey\Service\ResponseExportService;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\simple_survey\Entity\SimpleSurveyInterface;
use Drupal\simple_survey\Entity\SurveyQuestion;

/**
 * Class FeedbackjsController.
 *
 * @package Drupal\simple_survey\Controller
 */
class SimpleSurveyRestController extends ControllerBase {

  /**
   * The modules path.
   *
   * @var string
   */
  private $modulePath = '';

  /**
   * The modules machine name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Drupal\Component\Uuid\UuidInterface definition.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * Drupal\Core\Logger\LoggerChannelFactory definition.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected $loggerFactory;

  /**
   * Drupal date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * Response export service.
   *
   * @var \Drupal\simple_survey\BigQueryConnect
   */
  protected $bigQuery;

  /**
   * Response export service.
   *
   * @var \Drupal\simple_survey\ResponseExportService
   */
  protected $responseExport;

  /**
   * The Drupal Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Simple Survery Controller constructor.
   *
   * @todo Look into creating a service to handle DI. Doing a janky call in simple_survey.module
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The Drupal messenger service.
   * @param \Drupal\Component\Uuid\UuidInterface $uuid
   *   The Drupal UUID service.
   * @param Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The Drupal date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\simple_survey\BigQueryConnect $big_query
   *   The BigQuery Connection service.
   * @param \Drupal\simple_survey\ResponseExportService $response_export_service
   *   The response export service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The Drupal time service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager service.
   */
  public function __construct(
    MessengerInterface $messenger,
    UuidInterface $uuid,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $logger_factory,
    BigQueryConnect $big_query,
    ResponseExportService $response_export_service,
    TimeInterface $time,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->messenger = $messenger;
    $this->uuidService = $uuid;
    $this->dateFormatter = $dateFormatter;
    $this->loggerFactory = $logger_factory->get($this->moduleName);
    $this->bigQuery = $big_query;
    $this->responseExport = $response_export_service;
    $this->time = $time;
    $this->entityTypeManager = $entity_type_manager;

    $this->modulePath = drupal_get_path('module', $this->moduleName);
  }

  /**
   * Dependency Injection.
   *
   * @inheritDoc
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('uuid'),
      $container->get('date.formatter'),
      $container->get('logger.factory'),
      $container->get('simple_survey.big_query'),
      $container->get('simple_survey.response_export'),
      $container->get('datetime.time'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Generates a Cookie for tracking and CSRF.
   *
   * @return Symfony\Component\HttpFoundation\Response
   *   Returns a http response.
   */
  private function generateCookie(Request $request, string $cookieName) {
    $expireTime = $this->time->getRequestTime() + (60 * 60 * 24);
    $headerResponse = new Response();
    $cookie = new Cookie(
      $cookieName,
      $this->uuidService->generate(),
      $expireTime,
      '/',
      $request->getHost(),
      FALSE,
      FALSE,
      FALSE,
      'strict'
    );

    if ($cookie) {
      $headerResponse->headers->setCookie($cookie);
      $headerResponse->setStatusCode(Response::HTTP_OK);
      $headerResponse->setContent('Credential Set.');

      return $headerResponse;
    }
    else {
      $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
      $headerResponse->setContent('Invalid Request Received');

      return $headerResponse;
    }
  }

  /**
   * Checks if request to load a survey is valid and should be loaded displayed.
   *
   * @param Symfony\Component\HttpFoundation\Request $request
   *   The Drupal request.
   * @param Drupal\simple_survey\Entity\SimpleSurveyInterface $simple_survey
   *   The Simple survey config entity.
   *
   * @return Symfony\Component\HttpFoundation\Response
   *   The returned response.
   */
  public function validate(Request $request, SimpleSurveyInterface $simple_survey) {
    $headerResponse = new Response();

    $cookieName = $this->moduleName . $simple_survey->id();

    if (isset($_COOKIE[$cookieName . '.pause'])) {
      $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
      $headerResponse->setContent('Invalid Request Received');

      return $headerResponse;
    }
    elseif (!isset($_COOKIE[$cookieName])) {
      return $this->generateCookie($request, $cookieName);
    }
    else {
      $srids = $this->entityTypeManager->getStorage('survey_response')->getQuery()
        ->condition('cookie_uuid', Html::escape($_COOKIE[$cookieName]))
        ->condition('site_path', Html::escape($request->get('site_path')))
        ->count()
        ->execute();

      if ($srids > 0) {
        $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
        $headerResponse->setContent('Invalid Request Received');

        return $headerResponse;
      }
      else {
        $headerResponse->setStatusCode(Response::HTTP_OK);
        $headerResponse->setContent('Valid Request');

        return $headerResponse;
      }
    }
  }

  /**
   * Displays the questions as JSON.
   *
   * @param Drupal\simple_survey\Entity\SimpleSurveyInterface $simple_survey
   *   The Simple survey config entity.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Returns a JSON Response.
   */
  public function displayQuestions(SimpleSurveyInterface $simple_survey) {
    $output = [];

    $query = $this->entityTypeManager()->getStorage('survey_question')->getQuery();

    $sqids = $query->condition('survey', $simple_survey->id())
      ->execute();

    $questions = SurveyQuestion::loadMultiple($sqids);

    foreach ($questions as $question) {
      $genericId = 'sq' . $question->id();
      $output[$genericId] = [
        'startingQuestion' => $simple_survey->get('startingQuestion') == $question->id(),
        'inputType' => $question->get('input_type')->value,
        'displayAs' => $question->get('display_as')->value,
        'questionType' => $question->get('question_type')->value,
        'id' => $genericId,
        'text' => [
          'en' => $question->get('question')->value,
        ],
        'placeholder' => [
          'en' => $question->get('placeholder_text')->value ?: '',
        ],
      ];

      foreach ($question->get('next_question') as $reference) {
        $nextQuestion = 'sq' . $reference->entity->id();
        $output[$genericId]['dependencies']['next_question'] = $nextQuestion;
      }

      foreach ($question->get('branching') as $reference) {
        $selectedValue = $reference->question_option;

        $output[$genericId]['dependencies'][$selectedValue] = 'sq' . $reference->entity->id();
      }

      foreach ($question->get('options')->getValue() as $key => $value) {
        switch ($output[$genericId]['inputType']) {
          case 'select':
            $output[$genericId]['options']['en'] = $value['value'];
            break;

          case 'radio':
            $output[$genericId]['options'][] = [
              'id' => $genericId . '_' . $key,
              'name' => $genericId,
              'text' => [
                'en' => $value['value'],
              ],
              'value' => $value['value'],
            ];
            break;

          case 'checkbox':
            $output[$genericId]['options'][] = [
              'id' => $genericId . '_' . $key,
              'name' => $genericId . '[]',
              'text' => [
                'en' => $value['value'],
              ],
              'value' => $value['value'],
            ];
            break;
        }
      }
    }

    return new JsonResponse(json_encode($output), 200, [], TRUE);
  }

  /**
   * Displays the questions as JSON.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The drupal request.
   * @param Drupal\simple_survey\Entity\SimpleSurveyInterface $simple_survey
   *   The Simple survey config entity.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Returns a JSON Response.
   */
  public function submitForm(Request $request, SimpleSurveyInterface $simple_survey) {
    $headerResponse = new Response();
    $censor = new CensorWords();

    $cookieName = $this->moduleName . $simple_survey->id();

    if (!isset($_COOKIE[$cookieName]) || isset($_COOKIE[$cookieName . '.pause'])) {
      $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
      $headerResponse->setContent('Invalid Request Received');

      return $headerResponse;
    }

    $cookie_uuid = Html::escape($_COOKIE[$cookieName]);

    $surveyResponse = SurveyResponse::create();
    $surveyResponse->set('cookie_uuid', $cookie_uuid);
    $surveyResponse->set('survey', $$simple_survey);
    $surveyResponse->set('created', $this->time->getRequestTime());

    $surveyResponse->set('site_path', Html::escape($request->get('site_path')));
    $surveyResponse->set('langcode', Html::escape($request->get('language')));

    try {
      $surveyResponse->save();
      $questions = SurveyQuestion::loadMultiple();

      foreach ($questions as $question) {
        $profanityFilteredAnswer = $censor->censorString(Xss::filter($request->get('sq' . $question->id())));
        $questionAnswer = $profanityFilteredAnswer['clean'];

        if ($questionAnswer) {
          $surveyAnswer = SurveyAnswer::create();

          $surveyAnswer->set('parent_uuid', $surveyResponse->uuid());
          $surveyAnswer->set('answer', $questionAnswer);
          $surveyAnswer->set('question_id', $question->id());

          try {
            $surveyAnswer->save();
          }
          catch (EntityStorageException $e) {
            $this->loggerFactory->alert($e);

            $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
            $headerResponse->setContent('Invalid Request Received');

            return $headerResponse;
          }
        }
      }

      $headerResponse->setStatusCode(Response::HTTP_OK);
      $headerResponse->setContent('Credential Set.');

      return $headerResponse;
    }
    catch (EntityStorageException $e) {
      $this->loggerFactory->alert($e);

      $headerResponse->setStatusCode(Response::HTTP_NO_CONTENT);
      $headerResponse->setContent('Invalid Request Received');

      return $headerResponse;
    }
  }

}
