<?php

namespace Drupal\simple_survey\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\simple_survey\Entity\SurveyQuestion;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for the Example add and edit forms.
 */
class SimpleSurveyForm extends EntityForm {

  /**
   * Constructs an SimpleSurveyForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entityTypeManager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    MessengerInterface $messenger,
    ModuleHandlerInterface $moduleHandler) {
    $this->entityTypeManager = $entityTypeManager;
    $this->messenger = $messenger;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * Provides messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $simple_survey = $this->entity;

    // Survey Info tabs.
    $form['simple_survey_info_tabs'] = [
      '#type' => $this->moduleHandler->moduleExists('field_group') ? 'horizontal_tabs' : 'vertical_tabs',
      '#title' => $this->t('Simple feedback survey Information'),
      '#tree' => TRUE,
      '#group_name' => 'simple_survey_info_tabs',
    ];

    // Survey info tab.
    $form['simple_survey_info'] = [
      '#type' => 'details',
      '#title' => $this->t('Survey General Info'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#group' => 'simple_survey_info_tabs',
      '#group_name' => 'simple_survey_info',
    ];

    // Question list.
    $form['question_list_detail'] = [
      '#type' => 'details',
      '#title' => $this->t('Questions'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#group' => 'simple_survey_info_tabs',
      '#group_name' => 'question_list_detail',
    ];

    // Display config tab.
    $form['display_config_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Display Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#group' => 'simple_survey_info_tabs',
      '#group_name' => 'display_config_settings',
    ];

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $simple_survey->label(),
      '#description' => $this->t("Label for the Survey."),
      '#required' => TRUE,
      '#group' => 'simple_survey_info',
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $simple_survey->id(),
      '#machine_name' => [
        'exists' => [$this, 'exist'],
      ],
      '#disabled' => !$simple_survey->isNew(),
      '#group' => 'simple_survey_info',
    ];

    $form['description'] = [
      '#title' => $this->t('Description'),
      '#type' => 'textarea',
      '#default_value' => $simple_survey->getDescription(),
      '#group' => 'simple_survey_info',
    ];

    $form['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Debug Mode'),
      '#description' => $this->t('Form will ignore delay'),
      '#default_value' => $simple_survey->getDebug(),
      '#options' => [
        1 => "Yes",
        0 => "No",
      ],
      '#group' => 'simple_survey_info',
    ];

    $form['enableTranslation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Question Translation'),
      '#description' => $this->t('Use available translations'),
      '#default_value' => $simple_survey->get('enableTranslation'),
      '#options' => [
        1 => "Yes",
        0 => "No",
      ],
      '#group' => 'simple_survey_info',
    ];

    $form['survey_api_info'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Survey API Info'),
      '#group' => 'simple_survey_info',
    ];

    $form['survey_api_info']['basePath'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Survey API Base Path'),
      '#default_value' => $simple_survey->get('basePath'),
      '#disabled' => TRUE,
      '#name' => 'basePath',
    ];

    $previewUrl = '';

    if ($simple_survey->get('questionsPath')) {
      $previewUrl = Link::fromTextAndUrl('View Question API Data',
      Url::fromUri('internal:' . $simple_survey->get('questionsPath'), [
        'attributes' => [
          'target' => '_blank',
        ],
      ]))->toString();
    }

    $form['survey_api_info']['questionsPath'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Survey API Question Path'),
      '#default_value' => $simple_survey->get('questionsPath'),
      '#suffix' => $previewUrl,
      '#disabled' => TRUE,
      '#name' => 'questionsPath',
    ];

    $form['survey_api_info']['submitPath'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Survey API Submit Path'),
      '#default_value' => $simple_survey->get('submitPath'),
      '#disabled' => TRUE,
      '#name' => 'submitPath',
    ];

    $form['survey_api_info']['validationPath'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Survey API Validation Path'),
      '#default_value' => $simple_survey->get('validationPath'),
      '#disabled' => TRUE,
      '#name' => 'validationPath',
    ];

    $form['display_config_settings']['anchorTag'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Where will the form display on page?'),
      '#description' => $this->t('Decides where the survey will display on page. <strong>Body</strong> will cause the form to be sticky to a position on page. Any other value will cause it to attach to the element. Use <strong>#</strong> for IDs and <strong>.</strong> for classes.'),
      '#default_value' => $simple_survey->get('anchorTag') ?: 'body',
      '#name' => 'anchorTag',
    ];

    $form['display_config_settings']['attachStylesheet'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use internal form stylesheet'),
      '#description' => $this->t('Uses stylesheet that comes with Simple Survey'),
      '#default_value' => $simple_survey->get('attachStylesheet') ?: 1,
      '#options' => [
        1 => "Yes",
        0 => "No",
      ],
      '#name' => 'attachStylesheet',
    ];

    $form['display_config_settings']['display_filtering'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Display Filtering'),
      '#description' => $this->t('Settings for how the survey appears.'),
    ];

    $form['display_config_settings']['display_filtering']['displayFilterByPath'] = [
      '#type' => 'textarea',
      '#name' => 'displayFilterByPath',
      '#title' => $this->t('Display survey only on the listed paths.'),
      '#description' => $this->t('Enter a new path on each line. To include sub-paths add an asterisk Example: <br>/ <- home page only<br>/* whole site<br>/example <- only this page<br>/example/* <- only this page<br>!/example <- not on this page'),
      '#default_value' => $simple_survey->get('displayFilterByPath'),
      '#format' => 'plain_text',
    ];

    $form['display_config_settings']['display_timing'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Display Timing'),
    ];

    $form['display_config_settings']['display_timing']['displayDelayType'] = [
      '#type' => 'select',
      '#name' => 'displayDelayType',
      '#title' => $this->t('Delay based on position or time'),
      '#description' => $this->t('Toggles the Survey tool delay between none, position or time. Default is based on time'),
      '#default_value' => $simple_survey->get('displayDelayType') ?: 'time',
      '#options' => [
        "time" => "Time",
        "position" => "Position",
        "none" => "None",
      ],
    ];

    $form['display_config_settings']['display_timing']['delayPosition'] = [
      '#type' => 'radios',
      '#title' => $this->t('Page Display Position'),
      '#description' => $this->t('Point at which the form should display if using the position based display. Default is bottom'),
      '#default_value' => $simple_survey->get('delayPosition') ?: 'bottom',
      '#options' => [
        "bottom" => "Bottom",
        "top" => "Top",
        "middle" => "Middle",
      ],
      '#name' => 'delayPosition',
      '#states' => [
        'visible' => [
          ':input[name="displayDelayType"]' => ['value' => 'position'],
        ],
      ],
    ];

    $form['display_config_settings']['display_timing']['delay'] = [
      '#type' => 'number',
      '#title' => $this->t('Form Time Delay'),
      '#description' => $this->t('Amount of time before form displays'),
      '#default_value' => $simple_survey->get('delay') ?: 0,
      '#name' => 'delay',
      '#states' => [
        'visible' => [
          ':input[name="displayDelayType"]' => ['value' => 'time'],
        ],
      ],
    ];

    $form['display_config_settings']['display_timing']['closingDelay'] = [
      '#type' => 'number',
      '#title' => $this->t('Closing Time Delay'),
      '#description' => $this->t('Amount of time before form closes after completion'),
      '#default_value' => $simple_survey->get('closingDelay') ?: 7,
      '#name' => 'closingDelay',
    ];

    $form['display_config_settings']['useFontAwesome'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use FontAwesome'),
      '#description' => $this->t('If the site has font awesome available, use it'),
      '#default_value' => $simple_survey->get('useFontAwesome') ?: 0,
      '#options' => [
        1 => "Yes",
        0 => "No",
      ],
      '#name' => 'useFontAwesome',
    ];

    $form = $this->dataStorage($form, $form_state);

    if (!$simple_survey->isNew()) {
      $form['question_list_detail']['info'] = [
        '#type' => 'item',
        '#markup' => $this->t('<p>Select the starting question from the options below or edit questions from here.</p>'),
      ];

      $form['question_list_detail']['startingQuestion'] = [
        '#type' => 'table',
        '#header' => [
          $this->t('Title'),
          $this->t('Operations'),
        ],
        '#empty' => $this->t('There are no questions yet. <a href="@add-url">Add a question.</a>', [
          '@add-url' => Url::fromRoute('entity.simple_survey.survey_question.add_form', [
            'simple_survey' => $simple_survey->id(),
          ])->toString(),
        ]),
        '#tableselect' => TRUE,
        '#multiple' => FALSE,
        '#js_select' => FALSE,
        '#default_value' => $simple_survey->get('startingQuestion'),
      ];

      $query = $this->entityTypeManager->getStorage('survey_question')->getQuery();

      $sqids = $query->condition('survey', $simple_survey->id())
        ->execute();

      $survey_questions = SurveyQuestion::loadMultiple($sqids);

      foreach ($survey_questions as $id => $survey_question) {
        // Some table columns containing raw markup.
        $form['question_list_detail']['startingQuestion'][$id]['title'] = [
          '#plain_text' => $survey_question->question->value,
        ];

        // Operations (dropbutton) column.
        $form['question_list_detail']['startingQuestion'][$id]['operations'] = [
          '#type' => 'operations',
          '#links' => [],
        ];

        $form['question_list_detail']['startingQuestion'][$id]['operations']['#links']['edit'] = [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('entity.simple_survey.survey_question.edit_form', [
            'survey_question' => $id,
            'simple_survey' => $simple_survey->id(),
          ]),
        ];

        $form['question_list_detail']['startingQuestion'][$id]['operations']['#links']['delete'] = [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('entity.survey_question.delete_form', [
            'survey_question' => $id,
          ]),
        ];
      }
    }

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  private function dataStorage(array $form, FormStateInterface $form_state) {

    $form['simple_survey_data_storage'] = [
      '#type' => 'details',
      '#title' => $this->t('Data Storage Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#group' => 'simple_survey_info_tabs',
      '#group_name' => 'simple_survey_data_storage',
    ];

    $simple_survey = $this->entity;

    $form['bigQueryProjectId'] = [
      '#type' => 'textfield',
      '#title' => $this->t('BigQuery Project Id'),
      '#description' => $this->t('Project Id from Google Console'),
      '#default_value' => $simple_survey->get('bigQueryProjectId'),
      '#format' => 'plain_text',
      '#group' => 'simple_survey_data_storage',
    ];

    $form['bigQueryKeyfile'] = [
      '#type' => 'textarea',
      '#title' => $this->t('BigQuery Keyfile Text'),
      '#description' => $this->t('Keyfile provided from BigQuery'),
      '#default_value' => $simple_survey->get('bigQueryKeyfile'),
      '#group' => 'simple_survey_data_storage',
    ];

    $form['bigQueryTable'] = [
      '#type' => 'textfield',
      '#title' => $this->t('BigQuery Table'),
      '#description' => $this->t('Table from Google Console'),
      '#default_value' => $simple_survey->get('bigQueryTable'),
      '#format' => 'plain_text',
      '#group' => 'simple_survey_data_storage',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $simple_survey = $this->entity;

    $simple_survey->set('basePath', $this->t('/simple-survey/api/@simple_survey', [
      '@simple_survey' => $simple_survey->id(),
    ]));

    $simple_survey->set('questionsPath', Url::fromRoute('simple_survey.survey_questions', [
      'simple_survey' => $simple_survey->id(),
    ])->toString());

    $simple_survey->set('submitPath', Url::fromRoute('simple_survey.survey_submit', [
      'simple_survey' => $simple_survey->id(),
    ])->toString());

    $simple_survey->set('validationPath', Url::fromRoute('simple_survey.validate', [
      'simple_survey' => $simple_survey->id(),
    ])->toString());

    $status = $simple_survey->save();

    if ($status) {
      $this->messenger()->addMessage($this->t('Saved the %label Survey.', [
        '%label' => $simple_survey->label(),
      ]));
    }
    else {
      $this->messenger()->addMessage($this->t('The %label Survey was not saved.', [
        '%label' => $simple_survey->label(),
      ]), MessengerInterface::TYPE_ERROR);
    }

    $form_state->setRedirect('entity.simple_survey.collection');
  }

  /**
   * Helper function to check whether an Simple Survey configuration entity exists.
   */
  public function exist($id) {
    $entity = $this->entityTypeManager->getStorage('simple_survey')->getQuery()
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }

}
