<?php

namespace Drupal\simple_survey\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\simple_survey\Entity\SurveyAnswer;
use Drupal\simple_survey\Entity\SurveyQuestion;
use Drupal\simple_survey\Entity\SurveyResponse;

/**
 * Survey Response convert and export service.
 */
class ResponseExportService {

  /**
   * The module name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * SimpleSurvey config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $simpleSurveyConfig;

  /**
   * Drupal config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * Drupal logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactory
   */
  protected $logger;

  /**
   * Drupal messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Service config and DI.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Drupal Messenger service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Drupal logger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager service.
   */
  public function __construct(
    MessengerInterface $messenger,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $loggerFactory,
    ConfigFactoryInterface $configFactory,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->messenger = $messenger;
    $this->configFactory = $configFactory;
    $this->dateFormatter = $dateFormatter;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $loggerFactory->get($this->moduleName);

    $this->simpleSurveyConfig = $this->configFactory->getEditable($this->moduleName . '.settings');
  }

  /**
   * Builds the response header.
   *
   * @return array
   *   Returns an array of header data.
   */
  private function getResponseHeader() {
    $headerArray = [
      'drupal_id' => 'Drupal ID',
      'cookie_uuid' => 'User',
      'site_path' => 'Page Submitted On',
      'langcode' => 'Language',
    ];

    $query = $this->entityTypeManager->getStorage('survey_question')->getQuery();

    $nids = $query
      ->execute();

    $questions = SurveyQuestion::loadMultiple($nids);

    foreach ($questions as $question) {
      if ($question->get('question_type')->value == 'normal') {
        $headerArray[$question->id()] = str_replace(["\r", "\n"], '', strip_tags($question->get('question')->value));
      }
    }

    $headerArray['created'] = 'Submission was created';

    return $headerArray;
  }

  /**
   * Builds a response data rray.
   *
   * @return array
   *   Returns an array of response data.
   */
  private function getResponseData() {
    $responseOutput = [];
    $query = $this->entityTypeManager->getStorage('survey_response')->getQuery();

    $srids = $query
      ->sort('created', 'DESC')
      ->execute();

    $survey_responses = SurveyResponse::loadMultiple($srids);

    foreach ($survey_responses as $survey_response) {
      $currentResponse = [
        'drupal_id' => $survey_response->id(),
        'cookie_uuid' => $survey_response->get('cookie_uuid')->value,
        'site_path' => $survey_response->get('site_path')->value,
        'langcode' => $survey_response->get('langcode')->value,
        'created' => $this->dateFormatter->format($survey_response->get('created')->value, 'custom', 'Y-m-d H:i:s'),
      ];

      $query = $this->entityTypeManager->getStorage('survey_answer')->getQuery();

      $saids = $query
        ->condition('parent_uuid', $survey_response->get('uuid')->value)
        ->execute();

      $survey_answers = SurveyAnswer::loadMultiple($saids);

      foreach ($survey_answers as $survey_answer) {
        $currentResponse[$survey_answer->get('question_id')->value] = $survey_answer->get('answer')->value;
      }

      $responseOutput[] = $currentResponse;
    }

    return $responseOutput;
  }

  /**
   * Converts survey responses to CSV.
   *
   * @param bool $collectOutput
   *   Flag to collect output.
   *
   * @return string
   *   Returns converted data.
   */
  public function convertDataToCsv($collectOutput = TRUE) {
    $csv = '';
    $responseData = $this->getResponseData();
    $headerArray = $this->getResponseHeader();
    $sanitizedHeaderArray = [];

    foreach ($headerArray as $item) {
      $sanitizedHeaderArray[] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', $item)));
    }

    $fp = fopen('php://temp', 'r+');
    fputcsv($fp, $sanitizedHeaderArray);

    foreach ($responseData as $response) {
      $output = [];

      foreach ($headerArray as $key => $label) {
        $output[] = isset($response[$key]) ? $response[$key] : '';
      }

      fputcsv($fp, $output);
    }

    if ($collectOutput) {
      rewind($fp);
      while (!feof($fp)) {
        $csv .= fread($fp, 8192);
      }

      fclose($fp);

      return $csv;
    }
    else {
      fclose($fp);
    }
  }

}
